package de.unikoblenz.fgbks.dmn.core.models;

import de.unikoblenz.fgbks.dmn.core.models.VerifierCollectionResult.Builder;
import de.unikoblenz.fgbks.dmn.core.verifier.*;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.RuleMap;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.slf4j.LoggerFactory;

public enum VerifierType {
  Identical("Checking for identical rules.", IdenticalRules.class),
  Subsumption("Checking for rules, which subsume other rules.", SubsumptionRules.class),
  Equivalent("Checking for synonyms in columns.", EquivalentRules.class),
  Overlap("Checking for overlapping rules.", OverlappingRules.class),
  Missing("Checking for missing rules.", MissingRules.class),
  PartialReduction(
      "Checking for partial reduction of rules (combination).", PartialReductionRules.class);

  private final Class<? extends AbstractVerifier> verifierClass;
  private String description;

  VerifierType(String description, Class<? extends AbstractVerifier> verifierClass) {
    this.description = description;
    this.verifierClass = verifierClass;
  }

  public String getDescription() {
    return description;
  }

  public Optional<VerifierResult> getResult(List<DmnDecision> dmnDecisionList, RuleMap ruleMap) {
    if (ruleMap == null) {
      ruleMap = RuleMap.createMapFromDmn(dmnDecisionList);
    }
    try {
      return Optional.of(
          verifierClass.newInstance().findValidationErrors(dmnDecisionList, ruleMap));
    } catch (Exception ex) {
      LoggerFactory.getLogger(VerifierType.class).warn(ex.getMessage());
      ex.printStackTrace();
    }
    return Optional.empty();
  }

  public static VerifierCollectionResult getAllResults(List<DmnDecision> dmnDecisionList) {
    RuleMap ruleMap = RuleMap.createMapFromDmn(dmnDecisionList);
    Builder builder = VerifierCollectionResult.getBuilder();
    Arrays.asList(values())
        .parallelStream()
        .forEach(
            verifierType ->
                verifierType
                    .getResult(dmnDecisionList, ruleMap)
                    .ifPresent(builder::addVerification));
    return builder.build();
  }
}
