package de.unikoblenz.fgbks.dmn.core.verifier;

import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.models.VerificationResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierType;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.RuleMap;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Type;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Value;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.logging.Logger;
import org.camunda.bpm.dmn.engine.DmnDecision;

public abstract class AbstractVerifier {

  protected static final Logger LOGGER = Logger.getLogger(AbstractVerifier.class.getSimpleName());

  private final VerifierType verifierType;
  private VerifierResult.Builder verifierResultBuilder;
  protected List<DmnDecision> dmnDecisionList;
  protected RuleMap ruleMap;

  protected AbstractVerifier(VerifierType verifierType) {
    this.verifierType = Objects.requireNonNull(verifierType);
  }

  public final VerifierResult findValidationErrors(
      final List<DmnDecision> dmnDecisionList, final RuleMap ruleMap) {
    LOGGER.info("Start verifyDecision " + this.getClass().getSimpleName());
    long start = System.currentTimeMillis();
    this.dmnDecisionList = new ArrayList<>(dmnDecisionList);
    this.ruleMap = ruleMap;
    verifierResultBuilder = VerifierResult.getBuilder().withVerifierType(verifierType);
    beforeVerifyDecision();
    for (DmnDecision d : dmnDecisionList) {
      verifyDecision(d);
    }
    afterVerifyDecision();
    long fin = System.currentTimeMillis();
    LOGGER.info(
        "Finish verifyDecision "
            + this.getClass().getSimpleName()
            + ". "
            + "Time: "
            + (fin - start));
    return verifierResultBuilder.build();
  }

  protected abstract void beforeVerifyDecision();

  protected abstract void verifyDecision(DmnDecision dmnDecision);

  protected abstract void afterVerifyDecision();

  protected final void addVerification(VerificationResult verificationResult) {
    verifierResultBuilder.addVerificationResult(Objects.requireNonNull(verificationResult));
  }

  protected boolean checkDifferentConclusion(
      String decisionKey, List<RuleIdentifier> currentRuleIdentifiers) {
    List<Type> outputCols =
        ruleMap.getAllOutputTypesFromDecisionKey(Objects.requireNonNull(decisionKey));
    for (Type outputType : outputCols) {
      List<Value> values =
          ruleMap.getValuesFromOutputType(
              outputType, Objects.requireNonNull(currentRuleIdentifiers));
      if (values.stream().map(Value::getBoundary).distinct().limit(2).count() > 1) {
        return true;
      }
    }
    return false;
  }
}
