package de.unikoblenz.fgbks.dmn.core.verifier;

import de.unikoblenz.fgbks.base.wordnet.WordnetService;
import de.unikoblenz.fgbks.dmn.core.models.VerificationResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierType;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.DataType;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Type;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Value;
import java.util.List;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class EquivalentRules extends AbstractVerifier {

  private static final Logger LOGGER = Logger.getLogger(EquivalentRules.class.getSimpleName());

  private WordnetService wns = WordnetService.getInstance();

  public EquivalentRules() {
    super(VerifierType.Equivalent);
  }

  @Override
  protected void beforeVerifyDecision() {}

  @Override
  protected void verifyDecision(DmnDecision d) {
    if (d.isDecisionTable()) {
      for (Type t : ruleMap.getAllTypesFromDecisionKey(d.getKey())) {
        // only check strings for equvalent rules (input + output)
        if (t.getDataType() == DataType.STRING) {
          // ensure expression is not null and not empty
          List<Value> values =
              ruleMap
                  .getValuesFromType(t)
                  .stream()
                  .filter(
                      v ->
                          v.getOrgExpression() != null
                              && !v.getOrgExpression().isEmpty()
                              && !v.getOrgExpression().equals("\"\""))
                  .collect(Collectors.toList());
          // Check all combinations for synonyms, if found -> add both rules to the message
          for (int i = 0; i < values.size() - 1; i++) {
            for (int u = i + 1; u < values.size(); u++) {
              String val1 = values.get(i).getOrgExpression().replace("\"", "");
              String val2 = values.get(u).getOrgExpression().replace("\"", "");
              if (!val1.equals(val2) && wns.areNounsSynonyms(val1, val2)) {
                addVerification(
                    VerificationResult.getBuilder()
                        .addRule(values.get(i).getRuleIdentifier())
                        .addRule(values.get(u).getRuleIdentifier())
                        .withMessage(
                            "In rule (%d) string \"%s\" and in rule (%d) string \"%s\": equal meaning? Are they synonyms?",
                            values.get(i).getRuleIdentifier().getRowNumber(),
                            val1,
                            values.get(u).getRuleIdentifier().getRowNumber(),
                            val2)
                        .build());
              }
            }
          }
        }
      }
    }
  }

  @Override
  protected void afterVerifyDecision() {}
}
