package de.unikoblenz.fgbks.dmn.core.verifier;

import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import de.unikoblenz.fgbks.dmn.core.models.VerificationResult;
import de.unikoblenz.fgbks.dmn.core.models.VerifierType;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Boundary;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Type;
import de.unikoblenz.fgbks.dmn.core.verifier.helper.Value;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import org.camunda.bpm.dmn.engine.DmnDecision;

public class MissingRules extends AbstractVerifier {

  public MissingRules() {
    super(VerifierType.Missing);
  }

  @Override
  protected void beforeVerifyDecision() {}

  @Override
  protected void verifyDecision(DmnDecision d) {
    if (d.isDecisionTable()) {
      List<Type> inputs = new ArrayList<>(ruleMap.getAllInputTypesFromDecisionKey(d.getKey()));
      List<Value[]> missingIntervals = new ArrayList<>();
      checkForMissingRules(d, inputs, 0, new Value[inputs.size()], missingIntervals, false);
      // remove duplicate rules
      for (int i = 0; i < missingIntervals.size(); i++) {
        if (missingIntervals.get(i) != null) {
          inner:
          for (int u = i + 1; u < missingIntervals.size(); u++) {
            if (missingIntervals.get(u) != null) {
              boolean isEqual;
              for (int x = 0; x < missingIntervals.get(i).length; x++) {
                if ((missingIntervals.get(i)[x].getOrgExpression() == null)
                    ^ (missingIntervals.get(u)[x].getOrgExpression() == null)) {
                  isEqual = false;
                } else {
                  isEqual =
                      (missingIntervals.get(i)[x].getOrgExpression() == null
                                  && missingIntervals.get(u)[x].getOrgExpression() == null
                              || missingIntervals
                                  .get(i)[x]
                                  .getOrgExpression()
                                  .equals(missingIntervals.get(u)[x].getOrgExpression()))
                          && missingIntervals.get(i)[x].getType()
                              == missingIntervals.get(u)[x].getType();
                }
                if (!isEqual) {
                  continue inner;
                }
              }
            }
            missingIntervals.set(u, null);
          }
        }
      }
      // TODO in later state: merge rules -> like partial Reduction
      // add errors for missing intervals
      for (Value[] missingInt :
          missingIntervals.stream().filter(Objects::nonNull).collect(Collectors.toList())) {
        StringBuilder sb = new StringBuilder("In table ");
        sb.append(d.getName()); // TODO: add real name
        sb.append(" the following rule is not defined: (");
        sb.append(
            Arrays.stream(missingInt)
                .map(Value::getOrgExpression)
                .collect(Collectors.joining("), (")));
        sb.append(")");
        addVerification(
            VerificationResult.getBuilder()
                .addRules(
                    Arrays.stream(missingInt)
                        .map(Value::getRuleIdentifier)
                        .collect(Collectors.toList()))
                .withMessage(sb.toString())
                .build());
      }
    }
  }

  private void checkForMissingRules(
      DmnDecision d,
      List<Type> inputs,
      int i,
      Value[] missingValues,
      List<Value[]> missingIntervals,
      boolean hasMissing) {
    if (i == inputs.size()) {
      if (hasMissing) {
        missingIntervals.add(missingValues.clone());
      }
    } else {
      List<Value> sortedBounds = ruleMap.getValuesFromInputType(inputs.get(i));
      sortedBounds.sort(Comparator.comparing(Value::getBoundary));

      Value lastBound = null;
      int z = 0;
      for (Value currentBound : sortedBounds) {
        boolean currentHasMissing = false;
        z++;
        Optional<Boundary> newBoundary = Optional.empty();
        String orgExpr = "";
        if (inputs.get(i).getDataType().isNumeric()) {
          if (lastBound == null) { // check current elements
            if (currentBound.getBoundary().getLowerBound() != Double.MIN_VALUE) {
              newBoundary = currentBound.getBoundary().getNewBoundaryLower();
            }
          } else {
            newBoundary = lastBound.getBoundary().getNewBoundaryBetween(currentBound.getBoundary());
          }
          if (newBoundary.isPresent()) {
            orgExpr = newBoundary.get().toString();
            currentHasMissing = true;
          }
        }
        if (newBoundary.isPresent()) {
          missingValues[i] =
              new Value(
                  newBoundary.get(),
                  orgExpr,
                  RuleIdentifier.getBuilder()
                      .withRowNumber(999)
                      .withDecision(d)
                      .withRuleId("newRule")
                      .build(),
                  inputs.get(i));
          checkForMissingRules(
              d, inputs, i + 1, missingValues, missingIntervals, hasMissing || currentHasMissing);
          missingValues[i] = null;
        }
        // check if the current rule is has missing ranges in next columns
        missingValues[i] = currentBound;
        checkForMissingRules(d, inputs, i + 1, missingValues, missingIntervals, hasMissing);
        missingValues[i] = null;

        // set next last bound
        if (lastBound == null
            || !lastBound
                .getBoundary()
                .isNumberInRange(
                    currentBound.getBoundary().getUpperBound(),
                    currentBound.getBoundary().getUpperBoundType())) {
          lastBound = currentBound;
        }
        // check last element in list
        newBoundary = Optional.empty();
        if (z == sortedBounds.size() && z != 1) {
          if (inputs.get(i).getDataType().isNumeric()) {
            if (lastBound.getBoundary().getUpperBound() != Double.MAX_VALUE) {
              newBoundary = lastBound.getBoundary().getNewBoundaryUpper();
              if (newBoundary.isPresent()) {
                orgExpr = newBoundary.get().toString();
                hasMissing = true;
              }
            }
          } else {
            newBoundary = Optional.of(lastBound.getBoundary());
            orgExpr = lastBound.getOrgExpression();
          }
          if (newBoundary.isPresent()) {
            missingValues[i] =
                new Value(
                    newBoundary.get(),
                    orgExpr,
                    RuleIdentifier.getBuilder()
                        .withRowNumber(999)
                        .withDecision(d)
                        .withRuleId("newRule")
                        .build(),
                    inputs.get(i));
            checkForMissingRules(d, inputs, i + 1, missingValues, missingIntervals, hasMissing);
            missingValues[i] = null;
          }
        }
      }
    }
  }

  @Override
  protected void afterVerifyDecision() {}
}
