package de.unikoblenz.fgbks.dmn.core.verifier.helper;

import de.unikoblenz.fgbks.dmn.core.models.RuleIdentifier;
import java.util.Objects;
import org.camunda.bpm.dmn.engine.DmnDecision;
import org.camunda.bpm.dmn.engine.impl.DmnExpressionImpl;

public abstract class Type {

  protected final String decisionKey;
  protected final String id;
  protected final DataType dataType;
  protected final String expression;

  public String getDecisionKey() {
    return decisionKey;
  }

  public String getId() {
    return id;
  }

  public DataType getDataType() {
    return dataType;
  }

  public String getExpression() {
    return expression;
  }

  protected Type(String decisionKey, String id, DataType dataType, String expression) {
    this.decisionKey = decisionKey;
    this.id = id;
    this.dataType = dataType;
    this.expression = expression;
  }

  public Value getValue(
      DmnExpressionImpl value, DmnDecision decision, int rowNumber, String ruleId) {
    return new Value(
        dataType.getBoundaryValue(value),
        value.getExpression(),
        RuleIdentifier.getBuilder()
            .withDecisionName(decision.getName())
            .withDecisionKey(decision.getKey())
            .withRowNumber(rowNumber)
            .withRuleId(ruleId)
            .withConditionId(value.getId())
            .withConditionName(value.getName())
            .build(),
        this);
  }

  public boolean isEqualExpression(Type type) {
    return this.expression.equals(type.expression);
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    Type type = (Type) o;
    return decisionKey.equals(type.decisionKey) && id.equals(type.id) && dataType == type.dataType;
  }

  @Override
  public int hashCode() {
    return Objects.hash(decisionKey, id, dataType);
  }
}
