let rootUrl = '';//http://localhost:8080/dmn-verifier-app/';
let dmnApi = rootUrl + 'rest/dmn/';

let $container = $('.editor-container');
let $tabs = $('.editor-tabs');
let CLASS_NAMES = {
  drd: 'dmn-icon-lasso-tool',
  decisionTable: 'dmn-icon-decision-table',
  literalExpression: 'dmn-icon-literal-expression'
};

$tabs.delegate('.tab', 'click', function (e) {
  var viewIdx = parseInt(this.getAttribute('data-id'), 10);
  var view = dmnModeler.getViews()[viewIdx];
  dmnModeler.open(view);
});

// modeler instance
var dmnModeler = new DmnJS({
  container: $container,
  height: 350,
  width: '100%',
  keyboard: {
    bindTo: window
  }
});

dmnModeler.on('views.changed', function (event) {
  var {views, activeView} = event;
  // clear tabs
  $tabs.empty();
  views.forEach(function (v, idx) {
    const className = CLASS_NAMES[v.type];
    var tab = $(`
            <div class="tab ${v === activeView ? 'active' : ''}" 
            data-id="${idx}" id="tab-dec-${v.element.id}">
              <span class="${className}"></span>
              ${v.element.name || v.element.id}
            </div>
          `);
    $tabs.append(tab);
  });
});

/**
 * Save diagram contents and print them to the console.
 */
function exportDiagram() {
  dmnModeler.saveXML({format: true}, function (err, xml) {
    if (err) {
      return console.log('could not save DMN 1.1 diagram');
    } else {
      console.log('saving dmn..');
    }
    $.ajax({
      url: dmnApi + 'setCurrentDmn',
      type: 'POST',
      contentType: 'text/xml',
      data: xml,
      success: function (data) {
        console.log('dmn saved');
      }
    });
  });
}

/**
 * Open diagram in our modeler instance.
 *
 * @param {String} dmnXML diagram to display
 */
function openDiagram(dmnXML) {
  // import diagram
  dmnModeler.importXML(dmnXML, function (err) {
    if (err) {
      return console.error('could not open DMN');
    }
    // fetch currently active view
    var activeView = dmnModeler.getActiveView();
    // apply initial logic in DRD view
    if (activeView.type === 'drd') {
      var activeEditor = dmnModeler.getActiveViewer();
      // access active editor components
      var canvas = activeEditor.get('canvas');
      // zoom to fit full viewport
      canvas.zoom('fit-viewport');
    }
  });
}

// load external diagram file via AJAX and open it
function refreshDmn() {
  $.get(dmnApi + 'getCurrentDmn/', openDiagram, 'text');
}

refreshDmn();

function addHighlightCss(ruleC) {
  $('#tab-dec-' + ruleC.rules[0].decisionKey).click();
  ruleC.rules.forEach(function (rule) {
    $('[data-row-id=' + rule.ruleId + ']').toggleClass('highlight');
  });
}
